"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Koa = require("koa");
const Router = require("koa-router");
const http_1 = require("http");
const methods = ['get', 'put', 'post', 'patch', 'delete', 'del'];
class WebServer extends Koa {
    constructor(pandoraContext) {
        super();
        this.pandoraContext = pandoraContext;
        this.router = new Router;
        this.server = http_1.createServer(this.callback());
        this.setup();
        this.use(this.router.routes());
        this.use(this.router.allowedMethods());
    }
    setup() {
        const router = this.router;
        const routes = this.getRoutes();
        for (const Route of routes) {
            const routePath = Route.route;
            const route = new Route(this);
            if (route.setup) {
                route.setup();
            }
            for (const method of methods) {
                if (route[method]) {
                    router[method](routePath, route[method].bind(route));
                }
            }
        }
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const { port, host } = this.getPort();
            yield new Promise((resolve, reject) => {
                this.server.listen({ port, host }, (err) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve();
                });
            });
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve) => {
                this.server.close(resolve);
            });
        });
    }
}
exports.WebServer = WebServer;
//# sourceMappingURL=WebServer.js.map