"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws_1 = require("ws");
const Actuator_1 = require("./Actuator");
const tailPkg = require("tail");
const readLastLines = require("read-last-lines");
const { Tail } = tailPkg;
class Stdout {
    constructor(app) {
        this.app = app;
        this.wss = new ws_1.Server({ noServer: true });
    }
    setup() {
        this.app.server.on('upgrade', (request, socket, head) => {
            const regRes = /^\/stdout\/(.*)$/.exec(request.url);
            if (!regRes) {
                socket.end();
                return;
            }
            const appName = regRes[1];
            this.wss.handleUpgrade(request, socket, head, (ws) => {
                this.handleConnection(appName, ws).catch((error) => {
                    ws.send(JSON.stringify({ type: 'error', error: String(error) }));
                    ws.terminate();
                }).catch(console.error);
            });
        });
    }
    handleConnection(appName, ws) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield Actuator_1.Actuator.get('/info?appName=' + encodeURIComponent(appName));
            if (!res.success) {
                throw new Error(res.message);
            }
            const app = res.data;
            const introspection = app[0].data;
            const stdoutLogPath = introspection.stdoutLogPath;
            const lastLines = yield readLastLines.read(stdoutLogPath, 100);
            const tail = new Tail(stdoutLogPath);
            ws.on('close', () => {
                tail.unwatch();
            });
            ws.send(JSON.stringify({ type: 'batch', content: lastLines }));
            tail.on('line', line => {
                ws.send(JSON.stringify({ type: 'line', content: line }));
            });
            tail.on('error', err => {
                console.error(err);
                ws.terminate();
            });
        });
    }
}
exports.Stdout = Stdout;
//# sourceMappingURL=Stdout.js.map